@@ -220,7 +220,7 @@ let adjust_sign_arity env evars p clauses =
     match clauses with
     | [] -> Context.Rel.nhyps p.program_sign
     | _ ->
-      List.fold_left (fun acc (_, lhs, rhs) ->
+      List.fold_left (fun acc (Pre_clause (_, lhs, rhs)) ->
           let len = List.length lhs in
           max acc len) 0 clauses
   in
@@ -238,7 +238,7 @@ let adjust_sign_arity env evars p clauses =
           user_err_loc (None, str "Too many patterns in clauses for this type")
     in aux evars max_args [] fullty
   in
-  let check_clause (loc, lhs, rhs) =
+  let check_clause (Pre_clause (loc, lhs, rhs)) =
     if List.length lhs < max_args then user_err_loc (loc, str "This clause has not enough arguments")
     else ()
   in List.iter check_clause clauses;
@@ -306,7 +306,11 @@ let env_of_rhs evars ctx env s lets =
  *   (List.rev ctx', id_pats ctx, ctx) *)
 
 let is_wf_ref id rec_type =
-  List.exists (function Some (Logical (_, id')) -> Id.equal id id' | _ -> false) rec_type
+  let aux = function 
+    | Some (Logical (nargs, (_, id'))) -> 
+      if Id.equal id id' then Some nargs else None 
+    | _ -> None
+  in CList.find_map aux rec_type
 
 let add_wfrec_implicits rec_type c =
   let open Glob_term in
@@ -319,15 +323,19 @@ let add_wfrec_implicits rec_type c =
           | GApp (fn, args) ->
             DAst.with_loc_val (fun ?loc gfn ->
                 match gfn with
-                | GVar fid when is_wf_ref fid rec_type ->
-                  let kind = Evar_kinds.{ qm_obligation = Define false;
-                                          qm_name = Anonymous;
-                                          qm_record_field = None }
-                  in
-                  let newarg = GHole (Evar_kinds.QuestionMark kind, Namegen.IntroAnonymous, None) in
-                  let newarg = DAst.make ?loc newarg in
-                  let args' = List.append (mapargs args) [newarg] in
-                  DAst.make ?loc (GApp (fn, args'))
+                | GVar fid -> 
+                  (match is_wf_ref fid rec_type with
+                  | exception Not_found -> maprec c
+                  | nargs -> 
+                    let kind = Evar_kinds.{ qm_obligation = Define false;
+                                            qm_name = Anonymous;
+                                            qm_record_field = None }
+                    in
+                    let newarg = GHole (Evar_kinds.QuestionMark kind, Namegen.IntroAnonymous, None) in
+                    let newarg = DAst.make ?loc newarg in
+                    let before, after = List.chop nargs (mapargs args) in
+                    let args' = List.append before (newarg :: after) in
+                    DAst.make ?loc (GApp (fn, args')))
                 | _ -> maprec c) fn
           | _ -> maprec c) c
     in aux c
@@ -367,7 +375,7 @@ let interp_program_body env sigma ctx data body ty =
           | Name na -> (na, mkRel i) :: subst
           | _ -> subst) 1 [] ctx
     in
-    let c = Vars.replace_vars subst c in
+    let c = Vars.replace_vars sigma subst c in
     let sigma =
       match ty with
       | None -> fst (Typing.type_of env sigma c)
@@ -644,7 +652,7 @@ let instance_of_pats env evars (ctx : rel_context) (pats : (int * bool) list) =
             if i'' == i' then Some (if hide then PHide i else PRel i)
             else None) pats)
       1 ctx'
-  in fst (rel_of_named_context ctx'), pats', pats''
+  in fst (rel_of_named_context evars ctx'), pats', pats''
 
 let push_rel_context_eos ctx env evars =
   if named_context env <> [] then
@@ -674,7 +682,7 @@ let push_named_context = List.fold_right push_named
 let check_unused_clauses env sigma cl =
   let unused = List.filter (fun (_, (_, used)) -> used = 0) cl in
   match unused with
-  | ((loc, lhs, _) as cl, _) :: cls ->
+  | (Pre_clause (loc, lhs, _) as cl, _) :: cls ->
     user_err_loc (loc, str "Unused clause " ++ pr_preclause env sigma cl)
   | [] -> ()
 
@@ -699,7 +707,9 @@ let compute_rec_type context programs =
       user_err_loc (None, Pp.str "Mutual well-founded definitions are not supported");
     let p = List.hd programs in
     match p.program_rec with
-    | Some (WellFounded (_, _, newinfo)) -> Some (Logical newinfo) :: context
+    | Some (WellFounded (_, _, id)) -> 
+      let nargs = Context.Rel.nhyps p.program_sign in
+      Some (Logical (nargs, id)) :: context
     | _ -> assert false
   end
 
@@ -1044,7 +1054,7 @@ let rec covering_aux env evars p data prev (clauses : (pre_clause * (int * int))
                            str " with problem " ++ pr_problem p env !evars prob ++
                            str " extpats " ++ pr_user_pats env !evars extpats);
   match clauses with
-  | ((loc, lhs, rhs), (idx, cnt) as clause) :: clauses' ->
+  | (Pre_clause (loc, lhs, rhs), (idx, cnt) as clause) :: clauses' ->
     if !Equations_common.debug then
       Feedback.msg_debug (str "Matching " ++ pr_user_pats env !evars (extpats @ lhs) ++ str " with " ++
                           pr_problem p env !evars prob);
@@ -1144,7 +1154,7 @@ let rec covering_aux env evars p data prev (clauses : (pre_clause * (int * int))
           | PRel i ->
             match prove_empty (env,evars) (pi1 prob) i with
             | Some (i, ctx, s) ->
-              Some (List.rev prev @ (((loc, lhs, rhs),(idx, cnt+1)) :: clauses'),
+              Some (List.rev prev @ ((Pre_clause (loc, lhs, rhs),(idx, cnt+1)) :: clauses'),
                     Compute (prob, [], ty, REmpty (i, s)))
             | None ->
               user_err_loc (loc,
@@ -1159,9 +1169,9 @@ let rec covering_aux env evars p data prev (clauses : (pre_clause * (int * int))
            | None ->
              user_err_loc
                (dummy_loc,
-                str"Clause " ++ pr_preclause env !evars (loc, lhs, Some rhs) ++
+                str"Clause " ++ pr_preclause env !evars (Pre_clause (loc, lhs, Some rhs)) ++
                 str" matched but its interpretation failed")
-           | Some s -> Some (List.rev prev @ ((loc,lhs,Some rhs),(idx, cnt+1)) :: clauses', s)))
+           | Some s -> Some (List.rev prev @ (Pre_clause (loc,lhs,Some rhs),(idx, cnt+1)) :: clauses', s)))
 
      | UnifFailure ->
        if !Equations_common.debug then Feedback.msg_debug (str "failed");
@@ -1422,7 +1432,7 @@ and interp_clause env evars p data prev clauses' path (ctx,pats,ctx' as prob)
       match cs with
       | [] -> cls
       | _ :: _ ->
-        [loc, lhs @ [DAst.make ?loc (PUVar (idref, Generated))], Some (Refine (cs, cls))]
+        [Pre_clause (loc, lhs @ [DAst.make ?loc (PUVar (idref, Generated))], Some (Refine (cs, cls)))]
     in
     let rec cls' n cls =
       let next_unknown =
@@ -1430,7 +1440,7 @@ and interp_clause env evars p data prev clauses' path (ctx,pats,ctx' as prob)
         let i = ref (-1) in fun () ->
           incr i; add_suffix str (string_of_int !i)
       in
-      List.map_filter (fun (loc, lhs, rhs) ->
+      List.map_filter (fun (Pre_clause (loc, lhs, rhs)) ->
         let oldpats, newpats = List.chop (List.length lhs - n) lhs in
         let newref, nextrefs =
           match newpats with hd :: tl -> hd, tl | [] -> assert false
@@ -1455,7 +1465,7 @@ and interp_clause env evars p data prev clauses' path (ctx,pats,ctx' as prob)
             | Some (Refine (cs', cls)) -> Some (Refine (cs', cls' (List.length cs' + n) cls))
             | _ -> rhs
           in
-          Some (loc, rev newlhs @ nextrefs, newrhs)
+          Some (Pre_clause (loc, rev newlhs @ nextrefs, newrhs))
         | _ ->
           CErrors.user_err ?loc
             (str "Non-matching clause in with subprogram:" ++ fnl () ++ int n ++
@@ -1528,7 +1538,7 @@ and interp_wheres env0 ctx evars path data s lets
 
     let is_rec = is_recursive id ([eqs], notations) in
     let p = interp_arity env evars ~poly:false ~is_rec ~with_evars:true notations eqs in
-    let clauses = List.map (interp_eqn env (data.notations @ notations) p ~avoid:Id.Set.empty) clauses in
+    let clauses = List.map (interp_eqn env !evars (data.notations @ notations) p ~avoid:Id.Set.empty) clauses in
     let sigma, p = adjust_sign_arity env !evars p clauses in
     let () = evars := sigma in
 
@@ -1603,7 +1613,7 @@ and covering ?(check_unused=true) env evars p data (clauses : pre_clause list)
        pr_problem p env !evars prob)
 
 let program_covering env evd data p clauses =
-  let clauses = List.map (interp_eqn (push_rel_context data.fixdecls env) 
+  let clauses = List.map (interp_eqn (push_rel_context data.fixdecls env) !evd
     data.notations p ~avoid:Id.Set.empty) clauses in
   let sigma, p = adjust_sign_arity env !evd p clauses in
   let () = evd := sigma in
