@@ -223,10 +223,26 @@ let find_rec_call is_rec sigma protos f args =
     else
       match alias with
       | Some (f',argsf) ->
+        let signlen = List.length sign in        
         let f', args' = Termops.decompose_app_vect sigma f' in
         let f' = EConstr.Unsafe.to_constr f' in
         if Constr.equal (head f') f then
-          Some (idx, arity, argsf, [], (args, args, []))
+          let sign, args = 
+            if signlen <= List.length args then
+              (* Exact or extra application *)
+              let indargs, rest = CList.chop signlen args in
+              let fargs = drop_last_n (List.length rest) args in
+              [], (fargs, indargs, rest)
+            else
+              (* Partial application *)
+              let sign = List.map EConstr.Unsafe.to_rel_decl sign in
+              let sign = substitute_args args sign in
+              let signlen = List.length sign in
+              let indargs = List.map (Constr.lift signlen) args @ Context.Rel.instance_list Constr.mkRel 0 sign in
+              let fargs = List.map (Constr.lift signlen) args @ Context.Rel.instance_list Constr.mkRel 0 sign in
+              sign, (fargs, indargs, [])
+          in
+          Some (idx, arity, argsf, sign, args)
         else None
       | None -> None
   in
@@ -289,7 +305,7 @@ let abstract_rec_calls sigma user_obls ?(do_subst=true) is_rec len protos c =
                hyps args
            in
            let fargs' = Constr.mkApp (f', Array.of_list fargs') in
-           let result = Termops.it_mkLambda_or_LetIn fargs' sign in
+           let result = Term.it_mkLambda_or_LetIn fargs' sign in
            let hyp =
              Term.it_mkProd_or_LetIn
                (Constr.mkApp (mkApp (mkRel (i + 1 + len + n + List.length sign), Array.of_list indargs'),
@@ -299,7 +315,8 @@ let abstract_rec_calls sigma user_obls ?(do_subst=true) is_rec len protos c =
            in
            let hyps = cmap_add hyp !occ hyps in
            let () = incr occ in
-           hyps, Term.applist (result, rest)
+           let c' = Term.applist (result, rest) in
+           hyps, c'
          | None ->
            let hyps =
              Array.fold_left (fun hyps arg -> let hyps', arg' = aux n env hyps arg in
@@ -573,10 +590,10 @@ let compute_elim_type env evd user_obls is_rec protos k leninds
   let nargs = List.length methods' + undefpreds + 1 in
   nargs, elimty
 
-let replace_vars_context inst ctx =
+let replace_vars_context sigma inst ctx =
   List.fold_right
     (fun decl (k, acc) ->
-      let decl' = map_rel_declaration (substn_vars k inst) decl in
+      let decl' = map_rel_declaration (substn_vars sigma k inst) decl in
       (succ k, decl' :: acc))
     ctx (1, [])
 
@@ -605,11 +622,12 @@ let map_proto evd recarg f ty =
   match recarg with
   | Some recarg ->
      let lctx, ty' = decompose_prod_assum evd ty in
+     (* Feedback.msg_debug Pp.(str"map_proto: " ++ Printer.pr_econstr_env (Global.env()) evd ty ++ str" recarg = " ++ int recarg); *)
      let app =
        let args = Termops.rel_list 0 (List.length lctx) in
        let before, after =
          if recarg == -1 then CList.drop_last args, []
-         else let bf, after = CList.chop (pred recarg) args in
+         else let bf, after = CList.chop recarg args in
               bf, List.tl after
        in
        applistc (lift (List.length lctx) f) (before @ after)
@@ -757,7 +775,9 @@ let subst_rec_programs env evd ps =
       let fn p oterm =
         match p.program_info.program_rec with
         | Some r ->
-          let recarg = match r with Structural _ -> None | WellFounded _ -> Some (-1) in
+          let recarg = match r with 
+            | Structural _ -> None 
+            | WellFounded _ -> Some (Context.Rel.nhyps p.program_info.program_sign - ctxlen) in
           let oterm = lift (List.length (pi1 p.program_prob) - ctxlen) oterm in
           Some (p.program_info.program_id, (recarg, oterm))
         | None -> None
@@ -1072,11 +1092,21 @@ let smash_ctx_map env sigma (l, p, r as m) =
 let pattern_instance ctxmap =
   List.rev_map pat_constr (filter_def_pats ctxmap)
 
-let computations env evd alias refine p eqninfo =
+type computation = Computation of
+  Equations_common.rel_context * EConstr.t *
+    alias option * EConstr.constr list * EConstr.t *
+    EConstr.t * (node_kind * bool) * Splitting.splitting_rhs *
+    ((EConstr.t * int list) *
+    alias option * Splitting.path * Equations_common.rel_context *
+    EConstr.t * EConstr.constr list * (EConstr.constr * (int * int)) option *
+    computation list)
+    list option
+
+let computations env evd alias refine p eqninfo : computation list =
   let { equations_prob = prob;
         equations_where_map = wheremap;
         equations_f = f } = eqninfo in
-  let rec program_computations env prob f alias fsubst refine p =
+  let rec program_computations env prob f alias fsubst refine p : computation list =
     computations env prob f alias fsubst (fst refine, false) p.program_splitting
   and computations env prob f alias fsubst refine = function
   | Compute (lhs, where, ty, c) ->
@@ -1148,8 +1178,8 @@ let computations env evd alias refine p eqninfo =
       Feedback.msg_debug Pp.(str"substituted: " ++ pr_splitting_rhs ~verbose:true envlhs envlhs evd lhs c' ty));
      let patsconstrs = pattern_instance ctx in
      let ty = substl inst ty in
-     [pi1 ctx, f, alias, patsconstrs, ty,
-      f, (Where, snd refine), c', Some wheres]
+     [Computation (pi1 ctx, f, alias, patsconstrs, ty,
+      f, (Where, snd refine), c', Some wheres)]
 
   | Split (_, _, _, cs) ->
     Array.fold_left (fun acc c ->
@@ -1176,12 +1206,12 @@ let computations env evd alias refine p eqninfo =
       (push_rel_context (pi1 lhs) env) evd info.refined_term);
      Feedback.msg_debug Pp.(str"At refine node, program term: " ++ Printer.pr_econstr_env 
       (push_rel_context (pi1 lhs) env) evd progterm); *)
-     [pi1 lhs, f, alias, patsconstrs, info.refined_rettyp, f, (Refine, true),
+     [Computation (pi1 lhs, f, alias, patsconstrs, info.refined_rettyp, f, (Refine, true),
       RProgram progterm,
       Some [(info.refined_term, filter), None, info.refined_path, pi1 info.refined_newprob,
             info.refined_newty, refinedpats,
             Some (mapping_constr evd info.refined_newprob_to_lhs c, info.refined_arg),
-            computations env info.refined_newprob (lift 1 info.refined_term) None fsubst (Regular, true) cs]]
+            computations env info.refined_newprob (lift 1 info.refined_term) None fsubst (Regular, true) cs])]
 
   in program_computations env prob f alias [] refine p
 
@@ -1410,7 +1440,7 @@ let all_computations env evd alias progs =
       computations env evd alias (kind_of_prog p.program_info,false) p in
     List.map (fun (p, unfp, prog, eqninfo) -> p, eqninfo, fn p unfp eqninfo) progs
   in
-  let rec flatten_comp (ctx, fl, flalias, pats, ty, f, refine, c, rest) =
+  let rec flatten_comp (Computation (ctx, fl, flalias, pats, ty, f, refine, c, rest)) =
     let rest = match rest with
       | None -> []
       | Some l ->
