@@ -94,11 +94,11 @@ let autorewrite_one b =
     match rules with
     | [] -> tclFAIL (str"Couldn't rewrite")
     | r :: rules ->
-       let global, _univs = Constr.destRef r.Autorewrite.rew_lemma in
+       let global, _univs = Constr.destRef (snd @@ Autorewrite.RewRule.rew_lemma r) in
        let tac =
          Proofview.tclBIND
          (pf_constr_of_global global)
-         (if r.Autorewrite.rew_l2r then Equality.rewriteLR else Equality.rewriteRL)
+         (if (Autorewrite.RewRule.rew_l2r r) then Equality.rewriteLR else Equality.rewriteRL)
        in
        Proofview.tclOR tac
          (fun e -> if !debug then Feedback.msg_debug (str"failed"); aux rules)
@@ -189,7 +189,7 @@ let mutual_fix li l =
       (fun sigma ->
         let nenv = Environ.reset_with_named_context (Environ.val_of_named_context sign) env in
         let (sigma, evs) = mk_holes nenv sigma types in
-        let evs = Array.map_of_list (Vars.subst_vars (List.rev li)) evs in
+        let evs = Array.map_of_list (Vars.subst_vars sigma (List.rev li)) evs in
         let types = Array.of_list types in
         let decl = (nas,types,evs) in
         let () = body := (fun i -> mkFix ((idx,i),decl)) in
@@ -209,7 +209,7 @@ let check_guard gls env sigma =
   let gl = Proofview.drop_state (List.hd gls) in
   try
     let evi = Evd.find sigma gl in
-    match evi.Evd.evar_body with
+    match Evd.evar_body evi with
     | Evd.Evar_defined b -> Inductiveops.control_only_guard (Evd.evar_env env evi) sigma b; true
     | Evd.Evar_empty -> true
   with Type_errors.TypeError _ -> false
